// Purpose: test if get_system_info suceeds.


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <OS.h>

//#define VERBOSE_MODE
#ifdef VERBOSE_MODE
#	define VERBOSE(x) printf x
#	define  get_system_info_etc get_system_info_verbose 
#else
#	define VERBOSE(x) ;
#	define get_system_info_etc get_system_info_laconic
#endif


void
get_system_info_laconic(system_info& systemInfo)
{
	get_system_info(&systemInfo);
}


void
get_system_info_verbose(system_info& systemInfo)
{
	get_system_info(&systemInfo);
	printf("------------ system info ---------------\n");
	printf("available memory: \t%ld MB\n", (systemInfo.max_pages * 4096) / (1024 * 1024));
	printf("used memory: \t\t%ld MB\n", (systemInfo.used_pages * 4096) / (1024 * 1024));
	printf("free memory: \t\t%ld MB\n",
		((systemInfo.max_pages - systemInfo.used_pages) * 4096) / (1024 * 1024));
	printf("-----------------------------------------\n");
}


int
main()
{
	system_info info;
	system_info tempInfo;
	get_system_info_etc(info);

	size_t chunkSize = 64 * 1024 * 1024;
	VERBOSE(("allocating %d MB\n", chunkSize / (1024 * 1024)));
	char* base = (char*)malloc(chunkSize);
	if (base) {
		VERBOSE(("before writing anything...\n"));
		get_system_info_etc(info);
		for (uint32 index = 0; index < chunkSize; index++)
			base[index] = 'a';
		VERBOSE(("after writing...\n"));
		get_system_info_etc(info);
		tempInfo.used_pages = info.used_pages;
		free(base);
	} else {
		fprintf(stderr, "malloc FAILED\n");
		exit(1);
	}

	VERBOSE(("memory is freed\n"));	

	get_system_info_etc(info);

	if (tempInfo.used_pages == info.used_pages) {
		// Still the get_system_info shows the allocated pages.

		printf("get_system_info: test FAILED\n");

		// Find how long the delay is.
		int numOfAdditionalGetSysInfoCalls = 0;
		bigtime_t start = system_time();
		bigtime_t stop = 0;
		while (tempInfo.used_pages == info.used_pages) {
			stop = 0;
			get_system_info(&info);
			stop = system_time();
			numOfAdditionalGetSysInfoCalls++;
			snooze(50);
		}

		// one last time
		get_system_info_verbose(info);

		printf("number of numOfAdditionalGetSysInfoCalls %d\n", numOfAdditionalGetSysInfoCalls);
		printf("delay is %ld µs\n", stop - start);
	} else
		printf("get_system_info: test PASSED\n");

	return 0;
}
